@@ -9,6 +9,7 @@ import (
 	"strings"
 	"time"
 
+	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	osv1 "github.com/openshift/api/operator/v1"
 	osnetnames "github.com/openshift/cluster-network-operator/pkg/names"
 	"github.com/pkg/errors"
@@ -91,6 +92,14 @@ func Add(mgr manager.Manager) error {
 	}
 	clusterInfo.SCCAvailable = sccAvailable
 
+	addMonitorServiceResources, err := IsMonitoringAvailable(clientset)
+	if err != nil {
+		// we don't want CNAO to fail only if monitoring cannot be activated.
+		addMonitorServiceResources = false
+		log.Printf("failed to check for availability of Monitoring namespace: %v", err)
+	}
+	clusterInfo.MonitoringAvailable = addMonitorServiceResources
+
 	return add(mgr, newReconciler(mgr, namespace, clusterInfo))
 }
 
@@ -340,14 +349,11 @@ func (r *ReconcileNetworkAddonsConfig) renderObjectsV1(networkAddonsConfig *cnao
 	}
 	objs = append([]*unstructured.Unstructured{applied}, objs...)
 
-	// Label objects with version of the operator they were created by
-	for _, obj := range objs {
-		labels := obj.GetLabels()
-		if labels == nil {
-			labels = map[string]string{}
-		}
-		labels[cnaov1.SchemeGroupVersion.Group+"/version"] = operatorVersionLabel
-		obj.SetLabels(labels)
+	err = updateObjectsLabels(networkAddonsConfig.GetLabels(), objs)
+	if err != nil {
+		log.Printf("failed to update objects labels: %v", err)
+		err = errors.Wrapf(err, "failed to update objects labels")
+		return objs, err
 	}
 
 	return objs, nil
@@ -498,6 +504,68 @@ func (r *ReconcileNetworkAddonsConfig) stopTrackingObjects() {
 	r.statusManager.SetFromPods()
 }
 
+func updateObjectsLabels(crLables map[string]string, objs []*unstructured.Unstructured) error {
+	var err error
+	for _, obj := range objs {
+		labels := obj.GetLabels()
+		if labels == nil {
+			labels = map[string]string{}
+		}
+		// Label objects with version of the operator they were created by
+		labels[cnaov1.SchemeGroupVersion.Group+"/version"] = operatorVersionLabel
+		labels[names.PROMETHEUS_LABEL_KEY] = names.PROMETHEUS_LABEL_VALUE
+		err = updateObjectTemplateLabels(obj, labels, []string{names.PROMETHEUS_LABEL_KEY})
+		if err != nil {
+			return err
+		}
+
+		appLabelKeys := []string{names.COMPONENT_LABEL_KEY, names.PART_OF_LABEL_KEY, names.VERSION_LABEL_KEY, names.MANAGED_BY_LABEL_KEY}
+
+		labels = updateLabelsFromCR(labels, crLables, appLabelKeys)
+		if err != nil {
+			return err
+		}
+		obj.SetLabels(labels)
+
+		err = updateObjectTemplateLabels(obj, labels, appLabelKeys)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func updateLabelsFromCR(labels, crLables map[string]string, appLabelKeys []string) map[string]string {
+	labels[names.COMPONENT_LABEL_KEY] = names.COMPONENT_LABEL_DEFAULT_VALUE
+	labels[names.MANAGED_BY_LABEL_KEY] = names.MANAGED_BY_LABEL_DEFAULT_VALUE
+	for _, key := range appLabelKeys {
+		if value, exist := crLables[key]; exist == true {
+			labels[key] = value
+		}
+	}
+
+	return labels
+}
+
+func updateObjectTemplateLabels(obj *unstructured.Unstructured, labels map[string]string, appLabelKeys []string) error {
+	kind := obj.GetKind()
+	if kind == "DaemonSet" || kind == "ReplicaSet" || kind == "Deployment" || kind == "StatefulSet" {
+		for _, key := range appLabelKeys {
+			if value, exist := labels[key]; exist == true {
+				err := unstructured.SetNestedField(obj.Object, value, "spec", "template", "metadata", "labels", key)
+				if err != nil {
+					log.Printf("failed to add relationship label %s: %v", key, err)
+					err = errors.Wrapf(err, "failed to add relationship labels")
+					return err
+				}
+			}
+		}
+	}
+
+	return nil
+}
+
 func collectContainersInfo(parentKind string, parentName string, containers []v1.Container) []cnao.Container {
 	containersInfo := []cnao.Container{}
 
@@ -539,6 +607,26 @@ func isSCCAvailable(c kubernetes.Interface) (bool, error) {
 	return isResourceAvailable(c, "securitycontextconstraints", "security.openshift.io", "v1")
 }
 
+// isMonitoringAvailable checks if we can deploy the monitoring component
+func IsMonitoringAvailable(c kubernetes.Interface) (bool, error) {
+	prometheusRuleResourceAvailable, err := isResourceAvailable(c, "customresourcedefinitions/prometheusrules.monitoring.coreos.com", "apiextensions.k8s.io", "v1")
+	if err != nil {
+		return false, errors.Wrap(err, "failed to check if prometheusRule resource is available")
+	}
+
+	serviceMonitorResourceAvailable, err := isResourceAvailable(c, "customresourcedefinitions/servicemonitors.monitoring.coreos.com", "apiextensions.k8s.io", "v1")
+	if err != nil {
+		return false, errors.Wrap(err, "failed to check if serviceMonitor resource is available")
+	}
+
+	if prometheusRuleResourceAvailable && serviceMonitorResourceAvailable {
+		return true, nil
+	}
+
+	log.Printf("will not deploy monitoring manifests: not all monitoring resources are available: %s: %v, %s, %v", monitoringv1.PrometheusRuleKind, prometheusRuleResourceAvailable, monitoringv1.ServiceMonitorsKind, serviceMonitorResourceAvailable)
+	return false, nil
+}
+
 func isRunningKubernetesNMStateOperator(c k8sclient.Client) (bool, error) {
 	deployments := &appsv1.DeploymentList{}
 	err := c.List(context.TODO(), deployments, k8sclient.MatchingLabels{"app": "kubernetes-nmstate-operator"})
