@@ -9,7 +9,6 @@ import (
 
 	"github.com/kubevirt/cluster-network-addons-operator/pkg/render"
 	"github.com/pkg/errors"
-	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
@@ -18,6 +17,11 @@ import (
 	cnao "github.com/kubevirt/cluster-network-addons-operator/pkg/apis/networkaddonsoperator/shared"
 )
 
+const (
+	WEBHOOK_REPLICAS     = int32(2)
+	WEBHOOK_MIN_REPLICAS = int32(1)
+)
+
 // renderNMState generates the manifests of NMState handler
 func renderNMState(conf *cnao.NetworkAddonsConfigSpec, manifestDir string, clusterInfo *ClusterInfo) ([]*unstructured.Unstructured, error) {
 	if conf.NMState == nil {
@@ -37,6 +41,8 @@ func renderNMState(conf *cnao.NetworkAddonsConfigSpec, manifestDir string, clust
 	data.Data["CertRotateInterval"] = conf.SelfSignConfiguration.CertRotateInterval
 	data.Data["CertOverlapInterval"] = conf.SelfSignConfiguration.CertOverlapInterval
 	data.Data["PlacementConfiguration"] = conf.PlacementConfiguration
+	data.Data["WebhookReplicas"] = WEBHOOK_REPLICAS
+	data.Data["WebhookMinReplicas"] = WEBHOOK_MIN_REPLICAS
 
 	log.Printf("NMStateOperator == %t", clusterInfo.NmstateOperator)
 	fullManifestDir := filepath.Join(manifestDir, "nmstate", "operand")
@@ -54,16 +60,23 @@ func renderNMState(conf *cnao.NetworkAddonsConfigSpec, manifestDir string, clust
 }
 
 func removeAppsV1Resource(ctx context.Context, client k8sclient.Client, name, namespace, kind string) []error {
+	return removeResource(ctx, client, types.NamespacedName{Namespace: namespace, Name: name}, schema.GroupVersionKind{Group: "apps", Version: "v1", Kind: kind})
+}
+
+func removeCoreV1Resource(ctx context.Context, client k8sclient.Client, name, namespace, kind string) []error {
+	return removeResource(ctx, client, types.NamespacedName{Namespace: namespace, Name: name}, schema.GroupVersionKind{Group: "", Version: "v1", Kind: kind})
+}
+
+func removeResource(ctx context.Context, client k8sclient.Client, key types.NamespacedName, gvk schema.GroupVersionKind) []error {
 	// Get existing
 	existing := &unstructured.Unstructured{}
-	gvk := schema.GroupVersionKind{Group: "apps", Version: "v1", Kind: kind}
 	existing.SetGroupVersionKind(gvk)
 
-	err := client.Get(ctx, types.NamespacedName{Name: name, Namespace: namespace}, existing)
+	err := client.Get(ctx, key, existing)
 
 	// if we found the object
 	if err == nil {
-		objDesc := fmt.Sprintf("(%s) %s/%s", gvk.String(), namespace, name)
+		objDesc := fmt.Sprintf("(%s) %s", gvk.String(), key)
 		log.Printf("Cleanup up %s Object", objDesc)
 
 		// Delete the object
@@ -77,31 +90,11 @@ func removeAppsV1Resource(ctx context.Context, client k8sclient.Client, name, na
 }
 
 func removeDaemonSetHandlerWorker(ctx context.Context, client k8sclient.Client) []error {
-	// Get existing
-	existing := &unstructured.Unstructured{}
-	gvk := schema.GroupVersionKind{Group: "apps", Version: "v1", Kind: "DaemonSet"}
-	existing.SetGroupVersionKind(gvk)
 	namespace := os.Getenv("OPERAND_NAMESPACE")
 	name := "nmstate-handler-worker"
+	kind := "DaemonSet"
 
-	err := client.Get(ctx, types.NamespacedName{Name: name, Namespace: namespace}, existing)
-	// if we found the object
-	if err == nil {
-		objDesc := fmt.Sprintf("(%s) %s/%s", gvk.String(), namespace, name)
-		log.Printf("Cleaning up %s Object", objDesc)
-
-		// Delete the object
-		err = client.Delete(ctx, existing)
-		if err != nil {
-			log.Printf("Failed Cleaning up %s Object", objDesc)
-			return []error{err}
-		}
-
-	} else if apierrors.IsNotFound(err) {
-		// object not found, no need for action.
-		return []error{}
-	}
-	return []error{}
+	return removeAppsV1Resource(ctx, client, name, namespace, kind)
 }
 
 func removeStandaloneHandler(ctx context.Context, client k8sclient.Client) []error {
@@ -128,6 +121,14 @@ func removeStandaloneCertManager(ctx context.Context, client k8sclient.Client) [
 	return removeAppsV1Resource(ctx, client, name, namespace, kind)
 }
 
+func removeConfig(ctx context.Context, client k8sclient.Client) []error {
+	namespace := os.Getenv("OPERAND_NAMESPACE")
+	name := "nmstate-config"
+	kind := "ConfigMap"
+
+	return removeCoreV1Resource(ctx, client, name, namespace, kind)
+}
+
 func cleanUpNMState(conf *cnao.NetworkAddonsConfigSpec, ctx context.Context, client k8sclient.Client, clusterInfo *ClusterInfo) []error {
 	if conf.NMState == nil {
 		return []error{}
@@ -135,6 +136,7 @@ func cleanUpNMState(conf *cnao.NetworkAddonsConfigSpec, ctx context.Context, cli
 
 	errList := []error{}
 	errList = append(errList, removeDaemonSetHandlerWorker(ctx, client)...)
+	errList = append(errList, removeConfig(ctx, client)...)
 	if clusterInfo.NmstateOperator {
 		errList = append(errList, removeStandaloneHandler(ctx, client)...)
 		errList = append(errList, removeStandaloneWebhook(ctx, client)...)
